package dev.fitko.fitconnect.core;

import com.nimbusds.jose.jwk.RSAKey;
import com.nimbusds.jwt.SignedJWT;
import dev.fitko.fitconnect.api.domain.model.event.EventPayload;
import dev.fitko.fitconnect.api.domain.model.metadata.Metadata;
import dev.fitko.fitconnect.api.domain.model.submission.Submission;
import dev.fitko.fitconnect.api.domain.model.submission.SubmissionForPickup;
import dev.fitko.fitconnect.api.domain.validation.ValidationResult;
import dev.fitko.fitconnect.api.services.Subscriber;
import dev.fitko.fitconnect.api.services.crypto.CryptoService;
import dev.fitko.fitconnect.api.services.events.EventLogService;
import dev.fitko.fitconnect.api.services.events.SecurityEventService;
import dev.fitko.fitconnect.api.services.submission.SubmissionService;
import dev.fitko.fitconnect.api.services.validation.ValidationService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Set;
import java.util.UUID;

public class SubmissionSubscriber implements Subscriber {

    private static final Logger LOGGER = LoggerFactory.getLogger(SubmissionSubscriber.class);

    private final SubmissionService submissionService;
    private final CryptoService cryptoService;
    private final ValidationService validationService;
    private final EventLogService eventLogService;
    private final SecurityEventService securityEventService;

    public SubmissionSubscriber(final SubmissionService submissionService,
                                final EventLogService eventLogService,
                                final CryptoService cryptoService,
                                final ValidationService validationService,
                                final SecurityEventService securityEventService) {
        this.submissionService = submissionService;
        this.eventLogService = eventLogService;
        this.cryptoService = cryptoService;
        this.validationService = validationService;
        this.securityEventService = securityEventService;
    }

    @Override
    public byte[] decryptStringContent(final RSAKey privateKey, final String encryptedContent) {
        return cryptoService.decryptBytes(privateKey, encryptedContent);
    }

    @Override
    public Set<SubmissionForPickup> pollAvailableSubmissions(final UUID destinationId, final int offset, final int limit) {
        return submissionService.pollAvailableSubmissions(destinationId, offset, limit).getSubmissions();
    }

    @Override
    public Submission getSubmission(final UUID submissionId) {
        return submissionService.getSubmission(submissionId);
    }

    @Override
    public String fetchAttachment(final UUID submissionId, final UUID attachmentId) {
        return submissionService.getAttachment(submissionId, attachmentId);
    }

    @Override
    public ValidationResult validateEventLog(final UUID caseId) {
        return ValidationResult.ok();
    }

    @Override
    public ValidationResult validateMetadata(final Metadata metadata) {
        return validationService.validateMetadataSchema(metadata);
    }

    @Override
    public ValidationResult validateHashIntegrity(final String originalHash, final byte[] data) {
        return validationService.validateIntegrity(originalHash, data);
    }

    @Override
    public void acceptSubmission(final EventPayload eventPayload) {
        final SignedJWT confirmedSubmissionEvent = securityEventService.createAcceptSubmissionEvent(eventPayload);
        eventLogService.sendEvent(eventPayload.getCaseId(), confirmedSubmissionEvent.serialize());
        LOGGER.info("CONFIRMED submission {} successfully", eventPayload.getSubmissionId());
    }

    @Override
    public void rejectSubmission(final EventPayload eventPayload) {
        final SignedJWT rejectSubmissionEvent = securityEventService.createRejectSubmissionEvent(eventPayload);
        eventLogService.sendEvent(eventPayload.getCaseId(), rejectSubmissionEvent.serialize());
        LOGGER.info("REJECTED submission {}", eventPayload.getSubmissionId());
    }
}
